{smcl}
{* *! version 7.0 01jul2011}{...}
{cmd:help dynsim}
{hline}

{title:Title}

{p2colset 5 18 20 2}{...}
{p2col :{hi:dynsim}    {hline 2}}Produce dynamic simulations of autoregressive relationships in ordinary least squares models{p_end}
{p2colreset}{...}


{title:Syntax}

{p 8 17 2}
{cmd:dynsim}{cmd:,} {cmd:ldv(}{it:varname}{cmd:)} {cmd:scen1(}{it:string}{cmd:)} [{cmd:scen2(}{it:string}{cmd:)} {cmd:scen3(}{it:string}{cmd:)} {cmd:scen4(}{it:string}{cmd:)} 
	{cmd:n(}{it:integer}{cmd:)} {cmd:sig(}{it:cilevel}{cmd:)} {cmd:shock(}{it:varname}{cmd:)} {cmd:shock_data(}{it:filename}{cmd:)} {cmd:shock_num(}{it:numlist}{cmd:)} {cmd:saving(}{it:string}{cmd:)} 
	{cmd:modify(}{it:varlist}{cmd:)} {cmd:inter(}{it:varlist}{cmd:)} {cmd:forecast(string)}]


{title: Description}

{pstd}{cmd:dynsim} uses King, Tomz and Wittenberg's (2000) Clarify statistical package to present long-term
dynamics for autoregressive series.  The user specifies a number (up to four) scenarios and the program estimates the value
of the predicted dependent variable (and confidence intervals) for each scenario for a number (user specified) of 
iterations.  The command is dynamic in that each iteration updates the value of the lagged dependent 
variable based on the predicted value from the previous iteration.  
	
{pstd}Available options include specifying the values of an exogenous variable ({opt shock}) to change with each iteration.  The program also effectively deals with interacting the shock variable with up to four other 
independent variables in the model.

{pstd}The user must have downloaded and installed the Clarify package ({it:{help estsimp}}, {it:{help setx}} and {it:{help simqi}}).  The user must estimate their model (via {opt estsimp} {opt regress}) prior to running dynsim.


{title:Options}

{phang}{opt ldv(varname)} specifies the name of the lagged dependent variable.

{phang}{opt scen1(string)} specifies the values of the variables used to generate the predicted values when t = 0.  At least one scenario must be given. The coding designation is identical to the options used in Clarify's {it:{help setx}} 
	(with the exception of setting the values to a specific observation).  At each subsequent iteration, these values will stay the same, except for the value of the lagged dependent variable, the shock variable 
	({opt shock}, if specified) and the interacted shock variable ({opt inter}, if specified).

{phang}{opt scen2-4(string)} are optional and are only used if more than one scenario is desired.  A maximum of 4 scenarios is 
	allowed.  These follow the same conventions as {opt scen1}.

{phang}{opt n(integer)} specifies the number of iterations (or time period) over which the program will generate the predicted value of the dependent 
	variable.  The default is 10.  
	
{phang}{opt sig(cilevel)} specifies the level of statistical significance of the confidence intervals (calculated via the percentile method).  This value must be between 10 and 99.99; see {manhelp level R}.  

{phang}{opt shock(varname)} this option allows the user to choose an independent variable (and its first {opt n} values) and have the 
	variable (and potentially different values) impact the scenarios at each simulation.  If this command is specified, the user must specify the {opt n} shock values through either a data set containing the variable ({opt shock_data}) 
	or a number list ({opt shock_num}).  For example, if the {opt shock_data(shock.dta)} is specified, then {cmd: dynsim} will read the first {opt n} values of the {opt shock} variable.  If {opt shock_num(numlist)} is specified, such as 
	0(10)100, the value of the {opt shock} variable will be 0 at time t+1, 10 at time t+2, and so on. 
	The number of values assigned to the shock variable must exceed the number of simulations.  If the shock 
	variable is interacted with another variable in the model, the user must also specify the name of the modifying variable ({opt modify}) and the interaction variable ({opt inter}).
	
{phang}{opt shock_data(string)} is one of two ways of specifying the shock values.  This must give the file 
	name, or must be located in the working directory.  The data set used to get the shock variable (called the shock 
	data set), must have at least the number of iterations specified in {opt n} and it must contain a variable with the same name as the shock variable ({opt shock}).  
	
{phang}{opt shock_num(numlist)} is the second way of specifying the shock values.  Any numlist is acceptable, as long as it contains at least {opt n} values; see {manhelp numlist P}.  

{phang}{opt mod:ify(varlist)} is the name of up to four variables that modify the relationship between the shock variable and the dependent variable.  If the shock variable is interacted with another variable in the model, 
	{cmd:dynsim} automatically updates the value of the interaction to be the product of the shock and modify variable at each iteration.  If the {opt inter} is specified, this must also be specified.  The same number of 
	variables must appear in the {opt inter} as {opt modify}.  The variables must also appear in the same order as in {cmd: estsimp}.

{phang}{opt int:er(varlist)} is the name of up to four interaction variables.  If {opt modify} is specified, this must be specified.  Much like {opt modify}, it must be in the same order as {cmd: estsimp}.  

{phang}{opt sav:ing(string)} is the name of the new data set created that contains the predicted values and CIs for each scenario.  It automatically replaces any data set with the same name, so change the name of the saving data set 
	if you do not want it replaced.

{phang}{opt fore:cast(string)} produces confidence intervals based on one of four options for calculating the conditional variance of a forecast: the {opt ae} calculates the standard errors analytically based on Enders' (2004: 79-81) 
	formula for the conditional variance of the forecast.  The {opt ag} calculates the standard errors analytically based on Greene's (2003: 571-580) formula for the conditional variance of a forecast.  The {opt se} and {opt sg} use the Enders and 
	Greene formulas, respectively, but use the simulations to produce {opt n} estimates of the conditional variance, which are then used to produce confidence intervals based on the percentile method.


{title:Examples}
{phang2}{cmd:. webuse grunfeld, clear}{p_end}
{phang2}{cmd:. xtset company year, yearly}{p_end}
{phang2}{cmd:. gen lag_invest = L.invest}{p_end}
{phang2}{cmd:. gen z = mvalue*kstock}{p_end}
{phang2}{cmd:. estsimp regress invest lag_invest mvalue kstock}{p_end}


{pstd}The following calculates 20 simulations of predicted values of invest (and 95% confidence intervals) based on two scenarios of the other variables' values and saves them into test_data.dta:{p_end}

{phang2}{cmd:. dynsim, ldv(lag_invest) scen1(lag_invest 139 mvalue 0 kstock 10) scen2(lag_invest 50 mvalue 20 kstock 100) n(20) sig(95) saving(test_data)}{p_end}

{pstd}We can view the returned matrices to examine the starting values of each scenario as well as the values for each iteration.  Here we present the matrices for the first scenario:{p_end}

{phang2}{cmd:. matrix list r(t0_s1)}{p_end}
{phang2}{cmd:. matrix list r(xc_s1)}{p_end}

{pstd}We can then load the data set, list the values, and graph them as a rcapsym figure:{p_end}

{phang2}{cmd:. preserve}{p_end}
	{phang2}{cmd:. use test_data, clear}{p_end}
	{phang2}{cmd:. list}{p_end}
	{phang2}{cmd:. twoway (rcapsym lower_1 upper_1 t, msymbol(O)) (rcapsym lower_2 upper_2 t, msymbol(Sh)), ytitle("Real Gross Investment") legend(label (1 "Scenario 1") label (2 "Scenario 2"))}{p_end}
{phang2}{cmd:. restore}{p_end}

{pstd}The following calculates 15 simulations of predicted values of invest (and 90% confidence intervals) based on two scenarios as well as a shock variable, which takes the values of the numlist at each subsequent simulation: {p_end}

{phang2}{cmd:. dynsim, ldv(lag_invest) scen1(lag_invest 139 mvalue 0 kstock 10) scen2(lag_invest 50 mvalue 20 kstock 100) n(15) sig(90) saving(test_data) shock(kstock) shock_num(0(1)20)}{p_end}

{pstd}The following calculates 10 simulations of predicted values of invest (and 99% confidence intervals) based on two scenarios as well as a shock variable, which takes the values of the numlist at each subsequent simulation; additionally, the value of inter changes based on the shock value (specified by the numlist) and the value of the modify variable. {p_end}
{pstd}First, re-estimate the model with the new interaction variable, z: {p_end}

{phang2}{cmd:. estsimp regress invest lag_invest mvalue kstock z, genname(k)}{p_end}

{phang2}{cmd:. dynsim, ldv(lag_invest) scen1(lag_invest 139 mvalue 0 kstock 10 z 0) scen2(lag_invest 50 mvalue 20 kstock 100 z 2000) n(10) sig(99) saving(test_data) shock(kstock) shock_num(0(1)20) modify(mvalue) inter(z)}{p_end}


{title:Saved results}

{pstd}
{cmd:dynsim} saves the following in {cmd:r()}:

{synoptset 15 tabbed}{...}
{p2col 5 15 19 2: Matrices}{p_end}
{synopt:{cmd:r(t0_s1)}}values of variables at time t = 0 for first scenario{p_end}
{synopt:{cmd:r(t0_s2)}}values of variables at time t = 0 for second scenario{p_end}
{synopt:{cmd:r(t0_s3)}}values of variables at time t = 0 for third scenario{p_end}
{synopt:{cmd:r(t0_s4)}}values of variables at time t = 0 for fourth scenario{p_end}
{synopt:{cmd:r(xc_s1)}}values of variables for first scenario{p_end}
{synopt:{cmd:r(xc_s2)}}values of variables for second scenario{p_end}
{synopt:{cmd:r(xc_s3)}}values of variables for third scenario{p_end}
{synopt:{cmd:r(xc_s4)}}values of variables for fourth scenario{p_end}
{p2colreset}{...}


{title:Authors}

{phang}Laron K. Williams, University of Missouri{break}
williamslaro@missouri.edu{p_end}

{phang}Guy D. Whiten, Texas A&M University{break}
whitten@polisci.tamu.edu{p_end}

{title:Citation}

{phang}For a more in-depth discussion of this program (including examples) or when citing this program, please view: "But Wait, There's More!  Maximizing Substantive Inferences in TSCS Models", paper presented at the 2010 Annual Meeting of the St. Louis Area Methods Meeting (SLAMM). {p_end}
